[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

[PATCH] xen-pciback: Allow setting PCI_MSIX_FLAGS_MASKALL too



When Xen domain configures MSI-X, the usual approach is to enable MSI-X
together with masking all of them via the config space, then fill the
table and only then clear PCI_MSIX_FLAGS_MASKALL. Allow doing this via
QEMU running in a stub domain.

Previously, when changing PCI_MSIX_FLAGS_MASKALL was not allowed, the
whole write was aborted, preventing change to the PCI_MSIX_FLAGS_ENABLE
bit too.

Note the Xen hypervisor intercepts this write anyway, and may keep the
PCI_MSIX_FLAGS_MASKALL bit set if it wishes to. It will store the
guest-requested state and will apply it eventually.

Signed-off-by: Marek Marczykowski-Górecki <marmarek@xxxxxxxxxxxxxxxxxxxxxx>
---
 drivers/xen/xen-pciback/conf_space_capability.c | 9 ++++++---
 1 file changed, 6 insertions(+), 3 deletions(-)

diff --git a/drivers/xen/xen-pciback/conf_space_capability.c 
b/drivers/xen/xen-pciback/conf_space_capability.c
index 5e53b4817f16..097316a74126 100644
--- a/drivers/xen/xen-pciback/conf_space_capability.c
+++ b/drivers/xen/xen-pciback/conf_space_capability.c
@@ -190,13 +190,16 @@ static const struct config_field caplist_pm[] = {
 };
 
 static struct msi_msix_field_config {
-       u16          enable_bit; /* bit for enabling MSI/MSI-X */
-       unsigned int int_type;   /* interrupt type for exclusiveness check */
+       u16          enable_bit;   /* bit for enabling MSI/MSI-X */
+       u16          allowed_bits; /* bits allowed to be changed */
+       unsigned int int_type;     /* interrupt type for exclusiveness check */
 } msi_field_config = {
        .enable_bit     = PCI_MSI_FLAGS_ENABLE,
+       .allowed_bits   = PCI_MSI_FLAGS_ENABLE,
        .int_type       = INTERRUPT_TYPE_MSI,
 }, msix_field_config = {
        .enable_bit     = PCI_MSIX_FLAGS_ENABLE,
+       .allowed_bits   = PCI_MSIX_FLAGS_ENABLE | PCI_MSIX_FLAGS_MASKALL,
        .int_type       = INTERRUPT_TYPE_MSIX,
 };
 
@@ -229,7 +232,7 @@ static int msi_msix_flags_write(struct pci_dev *dev, int 
offset, u16 new_value,
                return 0;
 
        if (!dev_data->allow_interrupt_control ||
-           (new_value ^ old_value) & ~field_config->enable_bit)
+           (new_value ^ old_value) & ~field_config->allowed_bits)
                return PCIBIOS_SET_FAILED;
 
        if (new_value & field_config->enable_bit) {
-- 
2.37.3




 


Rackspace

Lists.xenproject.org is hosted with RackSpace, monitoring our
servers 24x7x365 and backed by RackSpace's Fanatical Support®.